#!/usr/local/bin/perl

########################################################################
# edge2gv.perl:
# convert canonical edge representation to Geomview output
# input: lines of name/value pairs
# output: Geomview Command Language drawing great-circle arcs on globe
#  corresponding to latitude and longtitude of source and destination.
# tamara munzner
# 4/96
########################################################################


# "gv" or "oogl" 
# oogl can be turned into vrml easily, 
# while gv can be highlighted with mbone.perl
$outstyle = "gv";

# default "20", number of line segments in each arc.
$arcstep =  10;

# how far to draw arc from endpoint
# numbers are in spherical distance:
# 3.14 is halfway around sphere, so by default would always meet.
# if one of these number is smaller than half the spherical distance,
#  the tunnel will be drawn with a gap in the middle
# it's a way to see detail otherwise hidden by a bunch of long tunnels
# or set one end to 0 and the other to something small 
#  lets you see just sources or just sinks. 
$srcreach = 1.6;
$dstreach = 1.6;
$htmldoc = "default.html";

# override above defaults with command line name=val pairs
while (@ARGV) {
    ($lhs, $rhs) = split('=', shift(@ARGV));
    if (defined($lhs) && defined($rhs)) {
	eval "\$$lhs = \"$rhs\";";
    }
}
    
    

open(JUNK, ">junk");
open(ANCHORS, ">default.html");

open(TOGV, ">-");
if ($outstyle eq "gv") {
    print TOGV "(progn \n";
} else {
    print TOGV "{LIST \n";
}

$count = 0;

while (<STDIN>) {
    undef($srchost, $srclat, $srclon, $srcplace);
    undef($dsthost, $dstlat, $dstlon, $dstplace);
    undef($geom);
    # deal with a series of items that look like this:
    # ( "name=val", "name2=val2", ...)
    chop;
#    @line = split(',');
    @line = eval($_);
    foreach $item (@line) {
	($lhs,$rhs) = split('=', $item);
	if (defined($lhs) && defined($rhs)) {
	    eval "\$$lhs = \"$rhs\";";
	}
    }
    $color = "0 0 1" unless defined($color);
    $done = 0;
    if (($srclat == 0 &&  $srclon == 0) || ($dstlat == 0 && $dstlon ==0)) {
	printf JUNK "need latlon: $srcip $dstip $srchost $dsthost\n";
	$done = 1;
    }
    elsif (($srclat == $dstlat) && ($srclong == $dstlong)) {
	printf JUNK "indomain: $srcplace: $srchost $dsthost\n";
	$done = 1;
    }
    next if ($done == 1);
    printf JUNK "got: $srchost $srcplace $dsthost $dstplace\n";
    $vect = &makearc($srclat, $srclon, $dstlat, $dstlon, $color);
    $count++;
    ($srcphi, $srctheta) = (&tospherical($srclat,$srclon));
    ($dstphi, $dsttheta) = (&tospherical($dstlat,$dstlon));
    $linkname = "link$count:$srchost:$dsthost";
    $handlename = join("/",$srchost,$dsthost);
    $extra = sprintf("percent %4.2f", $traffic_percent) if defined($traffic_percent);
    $srcstr = sprintf("src $srchost $srcplace %.2f %.2f $extra",
		      $srclat, $srclon);
    $extra = sprintf("count %4.2f", $traffic_count) if defined($traffic_count);
    $dststr = sprintf("dst $dsthost $dstplace %.2f %.2f $extra", 
		      $dstlat, $dstlon);
    $str = &gvlink($vect, $handlename, $srcstr, $dststr, $linkname);
    $geom = "arcs" if !defined($geom);
    $groups{$geom} .= $str;
}
foreach $group (sort keys %groups) {
    &gvout($group, $groups{$group});
}

if ($outstyle eq "gv") {
    print TOGV ") \n";
} else {
    print TOGV "} \n";
}

close TOGV;
for ($i = 0; $i < 70; $i++) { print ANCHORS " <BR>"; }
close ANCHORS;
close JUNK;

########################################################################
#
# subroutines
#
########################################################################

sub gvlink {
    local($gstr, $handlename, $srcstr, $dststr, $linkname) = @_;
    undef(local($handle, $str));

    print ANCHORS "<hr><a name=$linkname> $srcstr<p>$dststr<p>\n" 
	unless ($outstyle eq "fast");

    if ($outstyle eq "fast") {
	return $gstr;
    }
    if ($outstyle eq "oogl") {
	$handle = "{ $gstr }"; 
    } elsif ($outstyle eq "gv") {
	#use handles, for highlighting selection with mbone.perl
	print TOGV "(hdefine geometry $handlename $gstr)\n";
	$handle = ": " . $handlename;
    }
    $str = " {LIST \
	     { COMMENT SRC MBONE {\"$srcstr\"}} \
	     { COMMENT DST MBONE {\"$dststr\"}} \
	     { COMMENT mbone HTITLE { $srcstr $dststr }} \
	     { COMMENT mbone HREF {\"$htmldoc#$linkname\"}} \
	     $handle \
	     } ";
}


sub gvout {
    local($name, $body) = @_;

    if ($outstyle eq "gv") {
	print TOGV "(geometry $name {LIST $body })\n";
    } else {
	print TOGV "{LIST $body }\n";
    }
}


# convert from lat/lon to spherical coords
sub tospherical {
    local($lat,$lon) = @_;
    local($phi, $theta);
    local($pi) = 3.14159;
    # rho = 1
    # convert from degrees to radians!!!

    # phi is 0 to 180, lat is 90 to -90
    # phi is 0 at wrong place: north pole instead of equator
    $phi = ($pi/180.0)*(90.0-$lat); 

    # theta is 0 to 360, lon is 180 to -180
    # theta is 0 at the right place: yz plane intersects greenwich england
    $theta = ($pi/180.0)*$lon; 
    ($phi, $theta);
}

# convert from spherical coords to cartesian coords on unit sphere
sub tocartesian {
    local($phi,$theta, $r) = @_;
    local($x,$y,$z);

#    printf "lat $lat lon $lon phi $phi theta $theta\n";
    $z = $r*cos($theta)*sin($phi);
    $x = $r*sin($theta)*sin($phi);
    $y = $r*cos($phi);
    ($x, $y, $z);
}


########################################################################
# construct the arcs
########################################################################

sub makepts {
    local($t, $ht, $theta, @basis) = @_;
    local($s, $c, $vpts, @vert);
    @b1 = @basis[0..2];
    @b2 = @basis[3..5];

    $s = sin($t);
    $c = cos($t);
    $vert[0] = $b1[0]*$c + $b2[0]*$s;
    $vert[1] = $b1[1]*$c + $b2[1]*$s;
    $vert[2] = $b1[2]*$c + $b2[2]*$s;
    @vert = &ptmul(1+$ht*sin($t/$theta*$pi), @vert, @vert);
    $vpts = " $vert[0] $vert[1] $vert[2] \n";
}

# geodesic on sphere code from Mille Niss's spherescribble program
# eqn from Cox & Eick InfoViz '95 paper: rho = 1+H*sin(x*pi)

sub makearc {
    local($srclat, $srclon, $dstlat, $dstlon, $color) = @_;
    undef(local(@b1, @b2, @p1, @p2));
    undef(local($vectpts));
    undef(local($t, $theta, $ht));
    undef(local($dstdist, $srcdist));
    local($pi) = 3.14159;
    local($maxht) = .5;
    local($minht) = .015;

    @p1 = &tocartesian(&tospherical($srclat, $srclon), 1);
    @p2 = &tocartesian(&tospherical($dstlat, $dstlon), 1);

    #find a basis for plane
    @b1 = @p1;
    $dot = &ptdot(@p1, @p2);
    $b2[0] = $p2[0] - $p1[0]*$dot;
    $b2[1] = $p2[1] - $p1[1]*$dot;
    $b2[2] = $p2[2] - $p1[2]*$dot;
    @b2 = &ptunit(@b2);
    #theta is length of geodesic segment between pts on sphere!
    $theta = &acos($dot);
    # maxht when theta is halfway across sphere
    $ht = $maxht*($theta/$pi);
    $ht = $minht if $ht < $minht;

    $srcdist = $srcreach < $theta ? $srcreach : $theta;
    if ($srcdist > 0) {
	for($i = 0; $i <= $arcstep-1; $i++) {
	    $t = $i*$srcdist/($arcstep-1);
	    $vectpts .= &makepts($t, $ht, $theta, @b1, @b2);
	}
    }
    $dstdist = $theta - $srcdist;
    $dstdist = $dstdist > $dstreach ? $dstreach : $dstdist;
    if ($dstdist > 0) {
	for($i = 0; $i <= $arcstep-1; $i++) {
	    $t = $theta - $i*$dstdist/($arcstep-1);
	    $vectpts .= &makepts($t, $ht, $theta, @b1, @b2);
	}
    }
    if (($srcdist && $srcdist < $srcreach && !$dstdist) || (!$srcdist && $dstdist)) {
	$vect = "VECT 1 $arcstep 1 $arcstep 1\n" . $vectpts . " $color 1 ";
    } elsif ($dstdist) {
	$vect = "VECT 2 " . $arcstep*2 . " 1 $arcstep $arcstep 1 0\n" . $vectpts . " $color 1 ";
    }
}

sub acos {
    local($num) = @_;
    atan2(sqrt(1-$num**2),$num);
}

sub ptdot {
    local($ax, $ay, $az, $bx, $by, $bz) = @_;
    $ax*$bx+$ay*$by+$az*$bz;
}

sub ptmul {
    local($s, $ax, $ay, $az, $bx, $by, $bz) = @_;
    $bx = $s*$ax;
    $by = $s*$ay;
    $bz = $s*$az;
    ($bx, $by, $bz);
}

sub ptlength {
    local($vx, $vy, $vz) = @_;
    sqrt( $vx*$vx + $vy*$vy + $vz*$vz );
}

sub ptunit {
    local(@v) = @_;
    local($len);

    $len = &ptlength(@v);
    &ptmul( 1./$len, @v, @v ) if( $len != 0. && $len != 1. );
}

sub ptadd {
    local ($ax, $ay, $az, $bx, $by, $bz) = @_;
    ($ax+$bx, $ay+$by, $az+$bz);
}
