#! /usr/local/bin/perl

# Parse input lines of AS ndp format of the form
# "src_asname=ANSBB-ASNNET-1","src_as=1327","dst_as=1327","dst_asname=ANSBB-ASNNET-1")  
# and produce .fe file
# use only nodes with at least $level peers (optional arg)

if (-t STDIN && @ARGV == 0) {
   print STDERR "Usage: $0 [-ndp] [minorder] [file.fe.dmp] [file.ndp] 
Reads a .ndp file, possibly preceded by an Evolver dump file.
Without -ndp, writes a new Evolver .fe file, incorporating vertex positions
from the given .fe.dmp file if any; other vertex positions are random.
With -ndp, writes a new .ndp file with \"position=x y z\" tags for each
node where the .fe.dmp file had one.
If integer \"minorder\" given, we emit only nodes with at least that many
neighbors in the graph.
Important: we assume (and can't check) that we're fed the same .ndp file
for each step of this process.
";
   exit(1);
}
$ndpout = 0;
$ndpout = 1, shift if $ARGV[0] eq "-ndp";
$level = $ARGV[0], shift if $ARGV[0] =~ /^\d+$/;

$vno = 0;
while(<>) {
    if(/^vertices/) {
	if($already) {
	   print STDERR "Sorry, .fe.dmp file must *precede* .ndp file!\n";
	   exit(1);
	}
	while(<>) {
	   last unless /^\s*\d/;
	   ($n, $xyz) = /(\S+)\s+(\S+\s+\S+\s+\S+)/;
	   $pos[$n] = $xyz;	# Fix position of vertex #n
	}
    } elsif (/^edges/) {
	while(<>) {
	   last unless /^\s*\d/;
	   ($n, $v0, $v1) = /(\S+)\s+(\S+)\s+(\S+)/;
	   $onedge[$v0] = 1; $onedge[$v1] = 1; # legit vertices
	}	    
    } elsif(/^\s*\(/) {
	undef($src_asname, $dst_asname, $src_as, $dst_as, $srchost, $dsthost);
	foreach $item (/\"([^\"]*)\"/g) {
	    $item =~ s/(.*)=(.*)/\$$1="$2"/;
	    eval($item);
	}
	$host1 = $src_asname || $src_as || $srchost;
	$host2 = $dst_asname || $dst_as || $dsthost;
	next if $host2 eq $host1;
	$host{$host1} = ++$vno, push(@hosts, $host1) unless $host{$host1};
	$host{$host2} = ++$vno, push(@hosts, $host2) unless $host{$host2};
	$order{$host1}++;
	$order{$host2}++;
	push(@arcs, "$host1 $host2");
	print if $ndpout;
    }
}

sub inorder {
   return  $order{$a} != $order{$b} ?
		$order{$b} <=> $order{$a} :
		$host{$a} <=> $host{$b};
}

@sortedhosts = sort inorder keys(%host);

$vno = 1;
foreach $host (@sortedhosts) {
    print "(\"nodename=$host\",\"position=$pos[$vno]\")\n"
	if $ndpout && defined($pos[$vno]) && $onedge[$vno];

    $hostname{$vno} = $host;
    $vertno{$host} = $vno++;
}

exit(0) if $ndpout;	# Just did all we needed to.

print "string

quantity knot energy global_method knot_energy
quantity hooke  ENERGY global_method hooke2_energy
define edge attribute hooke_size real
define vertex attribute node_charge real

vertices
";

foreach $h (@sortedhosts) {
  $vno = $vertno{$h};
  $at = $pos[$vno];
  $at = sprintf("%.3f %.3f %.3f", rand(1)-.5, rand(1)-.5, rand(1)-.5)
	unless defined($at) && $onedge[$vno];
  printf "%d\t%s node_charge %.1f // %s\n",
	$vertno{$h}, $at, $order{$h},
	$h
    if $order{$h} >= $level;
}

print "\nedges\n";

$e = 1;
foreach $ends (@arcs) {
  @ends = split(' ', $ends);
  $order0 = $order{ $ends[0] };
  $order1 = $order{ $ends[1] };
  $v0 = $vertno{$ends[0]};
  $v1 = $vertno{$ends[1]};
  $mycolor = 3;
  $mycolor = 2 if defined($pos[$v0]) && defined($pos[$v1]);
  printf "%d\t%s %s hooke_size %.2f color %d\n",  # and, "color N" if we have one.
	$e++,
	$v0, $v1,
	$order0 + $order1,
	$mycolor
	    if $order0 >= $level && $order1 >= $level;
}

print "

read
set edge density 0
hooke2_power := 1.5
hooke_modulus := 1
knot_power := 1
knot_modulus := 18
conj_grad on
";
